@@ -5,7 +5,7 @@
 #if !defined(SPAWN_DEBUG) || defined(_MSC_VER)
 #define PING()
 #else
-#define PING() fprintf (stderr, "%s:%s:%d\n", __FILE__, __FUNCTION__, __LINE__); fflush (stderr)
+#define PING() fprintf (stderr, "%s:%s:%d\n", __FILE__, _DBUS_FUNCTION_NAME, __LINE__); fflush (stderr)
 #endif
 
 #include <stdio.h>
@@ -105,7 +105,7 @@ _dbus_babysitter_new (void)
 
   old_refcount = _dbus_atomic_inc (&sitter->refcount);
 
-  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, __FUNCTION__);
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, _DBUS_FUNCTION_NAME);
 
   sitter->child_handle = NULL;
 
@@ -139,7 +139,7 @@ _dbus_babysitter_ref (DBusBabysitter *sitter)
 
   old_refcount = _dbus_atomic_inc (&sitter->refcount);
   _dbus_assert (old_refcount > 0);
-  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, __FUNCTION__);
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount+1, _DBUS_FUNCTION_NAME);
 
   return sitter;
 }
@@ -180,7 +180,7 @@ _dbus_babysitter_unref (DBusBabysitter *sitter)
 
   old_refcount = _dbus_atomic_dec (&sitter->refcount);
   _dbus_assert (old_refcount > 0);
-  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount-1, __FUNCTION__);
+  _dbus_babysitter_trace_ref (sitter, old_refcount, old_refcount-1, _DBUS_FUNCTION_NAME);
 
   if (old_refcount == 1)
     {
@@ -368,15 +368,16 @@ protect_argv (char  * const *argv,
 {
   int i;
   int argc = 0;
+  char **args = NULL;
 
   while (argv[argc])
     ++argc;
-  *new_argv = dbus_malloc ((argc + 1) * sizeof (char *));
-  if (*new_argv == NULL)
+  args = dbus_malloc ((argc + 1) * sizeof (char *));
+  if (args == NULL)
     return -1;
 
   for (i = 0; i < argc; i++)
-    (*new_argv)[i] = NULL;
+    (args)[i] = NULL;
 
   /* Quote each argv element if necessary, so that it will get
    * reconstructed correctly in the C runtime startup code.  Note that
@@ -413,11 +414,13 @@ protect_argv (char  * const *argv,
           p++;
         }
 
-      q = (*new_argv)[i] = dbus_malloc (len + need_dblquotes*2 + 1);
+      q = args[i] = dbus_malloc (len + need_dblquotes*2 + 1);
 
       if (q == NULL)
-        return -1;
-
+        {
+          dbus_free_string_array (args);
+          return -1;
+        }
 
       p = argv[i];
 
@@ -443,106 +446,155 @@ protect_argv (char  * const *argv,
       if (need_dblquotes)
         *q++ = '"';
       *q++ = '\0';
-      /* printf ("argv[%d]:%s, need_dblquotes:%s len:%d => %s\n", i, argv[i], need_dblquotes?"TRUE":"FALSE", len, (*new_argv)[i]); */
+      /* printf ("argv[%d]:%s, need_dblquotes:%s len:%d => %s\n", i, argv[i], need_dblquotes?"TRUE":"FALSE", len, (args)[i]); */
     }
-  (*new_argv)[argc] = NULL;
+  args[argc] = NULL;
+  *new_argv = args;
 
   return argc;
 }
 
-
-/* From GPGME, relicensed by g10 Code GmbH.  */
-static char *
-compose_string (char **strings, char separator)
+static dbus_bool_t
+build_commandline (char **argv, DBusString *result)
 {
-  int i;
-  int n = 0;
-  char *buf;
-  char *p;
-
-  if (!strings || !strings[0])
-    return 0;
-  for (i = 0; strings[i]; i++)
-    n += strlen (strings[i]) + 1;
-  n++;
-
-  buf = p = malloc (n);
-  if (!buf)
-    return NULL;
-  for (i = 0; strings[i]; i++)
-    {
-      strcpy (p, strings[i]);
-      p += strlen (strings[i]);
-      *(p++) = separator;
-    }
-  p--;
-  *(p++) = '\0';
-  *p = '\0';
-
-  return buf;
+  return _dbus_string_append_strings (result, argv, ' ');
 }
 
-static char *
-build_commandline (char **argv)
+static dbus_bool_t
+build_env_block (char** envp, DBusString *result)
 {
-  return compose_string (argv, ' ');
-}
+  if (!_dbus_string_append_strings (result, envp, '\0'))
+    return FALSE;
 
-static char *
-build_env_string (char** envp)
-{
-  return compose_string (envp, '\0');
+   /* We need a double `\0` to terminate the environment block.
+    * DBusString provides one `\0` after the length-counted data,
+    * so add one more. */
+   if (!_dbus_string_append_byte (result, '\0'))
+     return FALSE;
+
+  return TRUE;
 }
 
+/**
+ * Creates a process with arguments and environment variables
+ *
+ * @param name name of the program
+ * @param argv list of char* pointers for the arguments
+ * @param envp list of char pointers for the environment
+ * @param inherit_handles specifies whether handles should be inherited by the child process
+ * @param error the error to set, if NULL no error will be set
+ * @return #NULL if an error occurred, the reason is returned in \p error
+ * @note The call to GetLastError() after this function may not return the expected value.
+ */
 HANDLE
 _dbus_spawn_program (const char *name,
                      char      **argv,
-                     char      **envp)
+                     char      **envp,
+                     dbus_bool_t inherit_handles,
+                     DBusError  *error)
 {
   PROCESS_INFORMATION pi = { NULL, 0, 0, 0 };
   STARTUPINFOA si;
-  char *arg_string, *env_string;
-  BOOL result;
+  DBusString arg_string = _DBUS_STRING_INIT_INVALID;
+  DBusString env_block = _DBUS_STRING_INIT_INVALID;
+  BOOL result = FALSE;
+  char *env = NULL;
+
+  if (!_dbus_string_init (&arg_string) || !_dbus_string_init (&env_block))
+    {
+      _DBUS_SET_OOM (error);
+      goto out;
+    }
 
 #ifdef DBUS_WINCE
   if (argv && argv[0])
-    arg_string = build_commandline (argv + 1);
-  else
-    arg_string = NULL;
+    {
+      if (!build_commandline (argv + 1, &arg_string))
+        _DBUS_SET_OOM (error);
+        goto out;
+    }
 #else
-  arg_string = build_commandline (argv);
+  if (!build_commandline (argv, &arg_string))
+    {
+      _DBUS_SET_OOM (error);
+      goto out;
+    }
 #endif
-  if (!arg_string)
-    return INVALID_HANDLE_VALUE;
+  if (_dbus_string_get_length (&arg_string) == 0)
+    {
+      dbus_set_error (error, DBUS_ERROR_FAILED, "No arguments given to start '%s'", name);
+      goto out;
+    }
 
-  env_string = build_env_string(envp);
+  if (envp != NULL)
+    {
+      if (!build_env_block (envp, &env_block))
+        {
+          _DBUS_SET_OOM (error);
+          goto out;
+        }
+      /* env_block consists of '0' terminated strings */
+      env = _dbus_string_get_data (&env_block);
+    }
 
   memset (&si, 0, sizeof (si));
   si.cb = sizeof (si);
 
 #ifdef DBUS_ENABLE_VERBOSE_MODE
   {
-    char *s = compose_string (envp, ';');
-    _dbus_verbose ("spawning '%s'' with args: '%s' env: '%s'\n", name, arg_string, s);
-    free (s);
+    DBusString temp = _DBUS_STRING_INIT_INVALID;
+
+    if (!_dbus_string_init (&temp))
+    {
+      _DBUS_SET_OOM (error);
+      goto out;
+    }
+
+    if (!_dbus_string_append_strings (&temp, envp, ';'))
+      {
+        _dbus_string_free (&temp);
+        _DBUS_SET_OOM (error);
+        goto out;
+      }
+
+    _dbus_verbose ("spawning '%s'' with args: '%s' env: '%s'\n", name,
+                   _dbus_string_get_const_data (&arg_string),
+                   _dbus_string_get_const_data (&temp));
+    _dbus_string_free (&temp);
   }
 #endif
 
 #ifdef DBUS_WINCE
-  result = CreateProcessA (name, arg_string, NULL, NULL, FALSE, 0,
+  result = CreateProcessA (name, _dbus_string_get_const_data (&arg_string), NULL, NULL, FALSE, 0,
 #else
-  result = CreateProcessA (NULL, arg_string, NULL, NULL, FALSE, 0,
+  result = CreateProcessA (NULL,  /* no application name */
+                           _dbus_string_get_data (&arg_string),
+                           NULL, /* no process attributes */
+                           NULL, /* no thread attributes */
+                           inherit_handles, /* inherit handles */
+                           0, /* flags */
 #endif
-			   (LPVOID)env_string, NULL, &si, &pi);
-  free (arg_string);
-  if (env_string)
-    free (env_string);
-
+                           env, NULL, &si, &pi);
   if (!result)
-    return INVALID_HANDLE_VALUE;
+    {
+      _dbus_win_set_error_from_last_error (error, "Unable to start '%s' with arguments '%s'",
+                                           name, _dbus_string_get_const_data (&arg_string));
+      goto out;
+    }
 
-  CloseHandle (pi.hThread);
-  return pi.hProcess;
+out:
+  _DBUS_ASSERT_ERROR_XOR_BOOL (error, result);
+
+  _dbus_string_free (&arg_string);
+  _dbus_string_free (&env_block);
+
+  if (result)
+    {
+      CloseHandle (pi.hThread);
+      return pi.hProcess;
+    }
+
+  return NULL;
 }
 
 
@@ -596,6 +648,7 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
   HANDLE handle;
   int argc;
   char **my_argv = NULL;
+  DBusError local_error = DBUS_ERROR_INIT;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   _dbus_assert (argv[0] != NULL);
@@ -666,7 +719,7 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
   _dbus_verbose ("babysitter: spawn child '%s'\n", my_argv[0]);
 
   PING();
-  handle = _dbus_spawn_program (sitter->log_name, my_argv, (char **) envp);
+  handle = _dbus_spawn_program (sitter->log_name, my_argv, (char **) envp, FALSE, &local_error);
 
   if (my_argv != NULL)
     {
@@ -674,15 +727,28 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter           **sitter_p,
     }
 
   PING();
-  if (handle == INVALID_HANDLE_VALUE)
+  if (handle == NULL)
     {
-      sitter->child_handle = NULL;
-      sitter->have_spawn_errno = TRUE;
-      sitter->spawn_errno = GetLastError();
-      dbus_set_error_const (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
-                            "Failed to spawn child");
+      if (dbus_error_has_name (&local_error, DBUS_ERROR_NO_MEMORY))
+        {
+          sitter->child_handle = NULL;
+          sitter->have_spawn_errno = TRUE;
+          sitter->spawn_errno = ERROR_NOT_ENOUGH_MEMORY;
+          dbus_move_error (&local_error, error);
+        }
+      else
+        {
+          sitter->child_handle = NULL;
+          sitter->have_spawn_errno = TRUE;
+          sitter->spawn_errno = GetLastError();
+          dbus_set_error (error, DBUS_ERROR_SPAWN_EXEC_FAILED,
+                          "Failed to spawn child: %s", local_error.message);
+        }
+      dbus_error_free (&local_error);
       goto out0;
     }
+  else
+    dbus_error_free (&local_error);
 
   sitter->child_handle = handle;
 
