@@ -1,9 +1,8 @@
+#include <config.h>
 #include <dbus/dbus.h>
 
-#define DBUS_COMPILATION /* cheat and use dbus-sysdeps */
 #include <dbus/dbus-sysdeps.h>
 #include <dbus/dbus-spawn.h>
-#undef DBUS_COMPILATION
 #include <stdio.h>
 
 static void
@@ -17,7 +16,7 @@ main (int argc, char **argv)
 {
   char **argv_copy;
   int i;
-  DBusError error;
+  DBusError error = DBUS_ERROR_INIT;
   
   if (argc < 2)
     {
@@ -31,11 +30,15 @@ main (int argc, char **argv)
     argv_copy [i] = argv[i + 1];
   argv_copy[argc - 1] = NULL;
   
-  if (!_dbus_spawn_async_with_babysitter (NULL, argv_copy, NULL, setup_func, NULL, &error))
+  if (!_dbus_spawn_async_with_babysitter (NULL, argv_copy[0], argv_copy, NULL,
+                                          DBUS_SPAWN_NONE, setup_func, NULL,
+                                          &error))
     {
       fprintf (stderr, "Could not launch application: \"%s\"\n",
 	       error.message);
     }
-  
+
+  dbus_free(argv_copy);
+  dbus_shutdown ();
   return 0;
 }
