@@ -21,6 +21,7 @@
 
 #include <config.h>
 
+#include "dbus/dbus-connection-internal.h"
 #include "dbus/dbus-internals.h"
 
 #include <stdio.h>
@@ -494,7 +495,7 @@ main (int argc, char *argv[])
   /* Receive o.fd.Peer messages as normal messages, rather than having
    * libdbus handle them internally, which is the wrong thing for
    * a monitor */
-  dbus_connection_set_route_peer_messages (connection, TRUE);
+  _dbus_connection_set_builtin_filters_enabled (connection, FALSE);
 
   if (!dbus_connection_add_filter (connection, filter_func,
                                    _DBUS_INT_TO_POINTER (binary_mode), NULL))
